@@ -85,6 +85,7 @@ class ChatDownloader():
                  end_time=None,
                  max_attempts=15,
                  retry_timeout=None,
+                 interruptible_retry=True,
                  timeout=None,
                  inactivity_timeout=None,
                  max_messages=None,
@@ -128,6 +129,9 @@ class ChatDownloader():
             this to a negative number will wait for user input. Default is None
             (use exponential backoff, i.e. immediate, 1s, 2s, 4s, 8s, ...)
         :type retry_timeout: float, optional
+        :param interruptible_retry: Have the option to skip waiting and
+            immediately retry. Defaults to True
+        :type interruptible_retry: bool, optional
         :param timeout: Stop retrieving chat after a certain duration
             (in seconds), defaults to None
         :type timeout: float, optional
@@ -258,15 +262,8 @@ class ChatDownloader():
                     x, format_name=params['format'])
 
                 if params['output']:
-                    output_file = ContinuousWriter(
-                        params['output'], indent=params['indent'], sort_keys=params['sort_keys'], overwrite=params['overwrite'])
-
-                    if output_file.is_default():
-                        chat.callback = lambda item: output_file.write(
-                            chat.format(item), flush=True)
-                    else:
-                        chat.callback = lambda item: output_file.write(
-                            item, flush=True)
+                    chat.attach_writer(ContinuousWriter(
+                        params['output'], indent=params['indent'], sort_keys=params['sort_keys'], overwrite=params['overwrite']))
 
                 chat.site = site_object
 
@@ -355,7 +352,7 @@ def run(propagate_interrupt=False, **kwargs):
                 pass
         else:
             def callback(item):
-                safe_print(chat.format(item), flush=True)
+                chat.print_formatted(item)
 
         for message in chat:
             callback(message)
