@@ -27,7 +27,7 @@ from .debugging import (
 )
 
 
-def main():
+def main(cli_args=None):
 
     parser = argparse.ArgumentParser(
         description=__summary__,
@@ -97,6 +97,7 @@ def main():
         'Retry Arguments')  # what to do when an error occurs
     add_chat_param(retry_group, '--max_attempts', type=int)
     add_chat_param(retry_group, '--retry_timeout', type=float)
+    add_chat_param(retry_group, '--interruptible_retry', is_boolean_flag=True)
 
     termination_group = parser.add_argument_group('Termination Arguments')
     add_chat_param(termination_group, '--max_messages', type=int)
@@ -169,10 +170,10 @@ def main():
     parser._positionals.title = 'Mandatory Arguments'
     parser._optionals.title = 'General Arguments'
 
-    args = parser.parse_args()
+    args = parser.parse_args(args=cli_args)
 
     # Modify debugging args:
-    if args.testing: # (only for CLI)
+    if args.testing:  # (only for CLI)
         args.logging = 'debug'
         args.pause_on_debug = True
 
