@@ -17,7 +17,8 @@ from ..utils.core import (
 
 from ..errors import (
     SiteError,
-    VideoUnavailable
+    VideoUnavailable,
+    LoginRequired
 )
 
 from ..debugging import (log, debug_log)
@@ -90,8 +91,64 @@ class FacebookChatDownloader(BaseChatDownloader):
     }
 
     _TESTS = [
-        # Unavailable (private?)
-        # https://www.facebook.com/SRAVS.Gaming/videos/512714596679251/
+        {
+            'name': 'Get chat messages from past broadcast',
+            'params': {
+                'url': 'https://www.facebook.com/disguisedtoast/videos/3629284013844544/',
+                'max_messages': 10
+            },
+            'expected_result': {
+                'messages_condition': lambda messages: 0 <= len(messages) <= 10,
+                'error': LoginRequired
+            }
+        },
+
+        {
+            'name': 'Get chat messages from clip',
+            'params': {
+                'url': 'https://www.facebook.com/disguisedtoast/videos/1170480696709027/',
+                'max_messages': 10
+            },
+            'expected_result': {
+                'messages_condition': lambda messages: 0 <= len(messages) <= 10,
+                'error': LoginRequired
+            }
+        },
+
+        {
+            'name': 'Get chat messages from short video',
+            'params': {
+                'url': 'https://www.facebook.com/disguisedtoast/videos/333201981735004/',
+                'max_messages': 10
+            },
+            'expected_result': {
+                'messages_condition': lambda messages: 0 <= len(messages) <= 10,
+                'error': LoginRequired
+            }
+        },
+
+        {
+            'name': 'Get chat messages from long video',
+            'params': {
+                'url': 'https://www.facebook.com/disguisedtoast/videos/918814568681983/',
+                'max_messages': 10
+            },
+            'expected_result': {
+                'messages_condition': lambda messages: 0 <= len(messages) <= 10,
+                'error': LoginRequired
+            }
+        },
+
+        # Check for errors
+        {
+            'name': 'Video unavailable or private',
+            'params': {
+                'url': 'https://www.facebook.com/SRAVS.Gaming/videos/512714596679251/',
+            },
+            'expected_result': {
+                'error': (VideoUnavailable, LoginRequired),
+            }
+        },
     ]
 
     _VIDEO_PAGE_TAHOE_TEMPLATE = _FB_HOMEPAGE + \
@@ -106,7 +163,8 @@ class FacebookChatDownloader(BaseChatDownloader):
     _GRAPH_API = _FB_HOMEPAGE + '/api/graphql/'
     _VIDEO_URL_FORMAT = _FB_HOMEPAGE + '/video.php?v={}'
 
-    def _attempt_fb_retrieve(self, url, max_attempts, retry_timeout=None, fb_json=False, is_json=True, **post_kwargs):
+    def _attempt_fb_retrieve(self, url, program_params, fb_json=False, is_json=True, **post_kwargs):
+        max_attempts = program_params.get('max_attempts')
         for attempt_number in attempts(max_attempts):
             try:
                 response = self._session_post(url, **post_kwargs)
@@ -120,25 +178,23 @@ class FacebookChatDownloader(BaseChatDownloader):
                     return response.text
 
             except JSONDecodeError as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout,
+                self.retry(attempt_number, error=e, **program_params,
                            text='Unable to parse JSON: `{}`'.format(response.text))
 
             except RequestException as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout)
+                self.retry(attempt_number, error=e, **program_params)
 
     _VIDEO_TITLE_REGEX = r'<meta\s+name=["\'].*title["\']\s+content=["\']([^"\']+)["\']\s*/>'
 
     def _get_initial_info(self, video_id, params):
         info = {}
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         # TODO remove duplication - many similar methods
         json_data = self._attempt_fb_retrieve(
             self._VIDEO_PAGE_TAHOE_TEMPLATE.format(video_id),
-            max_attempts,
-            retry_timeout,
-            True,
+            params,
+            fb_json=True,
             headers=self._FB_HEADERS, data=self.data
         )
 
@@ -162,10 +218,13 @@ class FacebookChatDownloader(BaseChatDownloader):
                     break
 
                 except RequestException as e:
-                    self.retry(attempt_number, max_attempts, e, retry_timeout)
+                    self.retry(attempt_number, error=e, **params)
 
         instances = multi_get(json_data, 'jsmods', 'instances')
         if not instances:
+            if '/login/?next=' in str(multi_get(json_data, 'jsmods', 'require')):
+                raise LoginRequired('Login required')
+
             log('debug', 'Data: {}'.format(json_data))
             raise VideoUnavailable('Video unavailable (may be private)')
 
@@ -502,9 +561,9 @@ class FacebookChatDownloader(BaseChatDownloader):
             info['in_reply_to'] = FacebookChatDownloader._parse_live_stream_node(
                 in_reply_to)
 
-        time_in_seconds = info.get('time_in_seconds')
-        if time_in_seconds is not None:
-            info['time_text'] = seconds_to_time(time_in_seconds)
+        # time_in_seconds = info.get('time_in_seconds')
+        # if time_in_seconds is not None:
+        #     info['time_text'] = seconds_to_time(time_in_seconds)
 
         message = info.get('message')
         if message:
@@ -522,9 +581,6 @@ class FacebookChatDownloader(BaseChatDownloader):
         return info
 
     def _get_live_chat_messages_by_video_id(self, video_id, params):
-        max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
-
         buffer_size = 25  # max num comments returned by api call
         # cursor = ''
         variables = {
@@ -545,8 +601,7 @@ class FacebookChatDownloader(BaseChatDownloader):
 
             json_data = self._attempt_fb_retrieve(
                 self._GRAPH_API,
-                max_attempts,
-                retry_timeout,
+                params,
                 headers=self._FB_HEADERS, data=data
             )
 
@@ -569,7 +624,18 @@ class FacebookChatDownloader(BaseChatDownloader):
                 log('debug', 'No top level comments: {}'.format(json_data))
                 continue
 
-            edges = reversed(top_level_comments.get('edges'))  # reverse order
+            # Parse items:
+            parsed_items = []
+            for edge in top_level_comments.get('edges') or []:
+                node = edge.get('node')
+                if not node:
+                    log('debug', 'No node found in edge: {}'.format(edge))
+                    continue
+                parsed_items.append(
+                    FacebookChatDownloader._parse_live_stream_node(node))
+
+            # Sort items
+            parsed_items.sort(key=lambda x: x['timestamp'])
 
             # TODO - get pagination working
             # page_info = top_level_comments.get('page_info')
@@ -577,29 +643,20 @@ class FacebookChatDownloader(BaseChatDownloader):
             # has_next_page = page_info.get('has_next_page')
 
             num_to_add = 0
-            for edge in edges:
-                node = edge.get('node')
-                if not node:
-                    log('debug', 'No node found in edge: {}'.format(edge))
-                    continue
-
-                comment_id = node.get('id')
+            for item in parsed_items:
+                comment_id = item.get('message_id')
 
                 # remove items that have already been parsed
                 if comment_id in last_ids:
                     continue
 
                 last_ids.append(comment_id)
-
                 last_ids = last_ids[-buffer_size:]  # force x items
 
-                parsed_node = FacebookChatDownloader._parse_live_stream_node(
-                    node)
                 # TODO determine whether to add or not (message types/groups)
 
                 num_to_add += 1
-
-                yield parsed_node
+                yield item
 
             # got 25 items, and this isn't the first one
             if num_to_add >= buffer_size and not first_try:
@@ -613,9 +670,6 @@ class FacebookChatDownloader(BaseChatDownloader):
 
     def _get_chat_replay_messages_by_video_id(self, video_id, max_duration, params):
 
-        max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
-
         # useful tool (convert curl to python request)
         # https://curl.trillworks.com/
         # timeout_duration = 10  # TODO make this modifiable
@@ -645,20 +699,12 @@ class FacebookChatDownloader(BaseChatDownloader):
 
             json_data = self._attempt_fb_retrieve(
                 self._VOD_COMMENTS_API,
-                max_attempts,
-                retry_timeout,
-                True,
+                params,
+                fb_json=True,
                 headers=self._FB_HEADERS, params=request_params, data=self.data
             )
 
-            next_start_time = next_end_time
-
-            if next_start_time >= end_time:
-                return
-
-            payloads = multi_get(json_data, 'payload', 'ufipayloads')
-            if not payloads:
-                continue
+            payloads = multi_get(json_data, 'payload', 'ufipayloads') or []
 
             for payload in payloads:
                 time_offset = payload.get('timeoffset')
@@ -674,7 +720,8 @@ class FacebookChatDownloader(BaseChatDownloader):
                 # pinned_comments = ufipayload.get('pinnedcomments')
                 profile = try_get_first_value(ufipayload['profiles'])
 
-                text = comment['body']['text']  # safe_convert_text()
+                # TODO proper parsing
+                text = comment['body']['text']
 
                 temp = {
                     'author': {
@@ -687,6 +734,10 @@ class FacebookChatDownloader(BaseChatDownloader):
 
                 yield temp
 
+            if next_end_time >= end_time:
+                return
+            next_start_time = next_end_time
+
     def _get_chat_by_video_id(self, match, params):
         return self.get_chat_by_video_id(match.group('id'), params)
 
@@ -723,6 +774,9 @@ class FacebookChatDownloader(BaseChatDownloader):
 
     def generate_urls(self, **kwargs):
         max_attempts = 10
+        program_params = {
+            'max_attempts': max_attempts
+        }
 
         limit = 50  # amount_to_get
         step = 8
@@ -751,7 +805,7 @@ class FacebookChatDownloader(BaseChatDownloader):
                 data.update(self.data)
                 req = self._attempt_fb_retrieve(
                     'https://www.facebook.com/ajax/route-definition/',
-                    max_attempts,
+                    program_params,
                     is_json=False,
                     headers=self._FB_HEADERS, data=data
                 )
@@ -766,7 +820,7 @@ class FacebookChatDownloader(BaseChatDownloader):
                 data.update(self.data)
                 json_data = self._attempt_fb_retrieve(
                     self._GRAPH_API,
-                    max_attempts,
+                    program_params,
                     headers=self._FB_HEADERS, data=data
                 )
 
