@@ -12,7 +12,6 @@ from ..utils.core import (
     try_parse_json,
     try_get_first_value,
     get_title_of_webpage,
-    try_get,
     chunks,
     ensure_seconds,
     seconds_to_time,
@@ -55,7 +54,7 @@ class RedditChatDownloader(BaseChatDownloader):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
 
-        max_attempts = 10
+        max_attempts = 3
 
         for attempt_number in attempts(max_attempts):
             initial_data = self._session_get(self._REDDIT_HOMEPAGE).text
@@ -66,7 +65,8 @@ class RedditChatDownloader(BaseChatDownloader):
                 break
             else:
                 title = get_title_of_webpage(initial_data)
-                self.retry(attempt_number, max_attempts, text=title)
+                self.retry(attempt_number, text=title,
+                           max_attempts=max_attempts)
                 continue
 
         bearer = multi_get(info, 'user', 'session', 'accessToken')
@@ -126,9 +126,18 @@ class RedditChatDownloader(BaseChatDownloader):
                 'messages_condition': lambda messages: len(messages) > 0,
             }
         },
-
         {
-            'name': 'Chat replay with start and end times',
+            'name': 'Get chat messages from past broadcast #5',
+            'params': {
+                'url': 'https://www.reddit.com/lmvsbl',
+                'max_messages': 10
+            },
+            'expected_result': {
+                'messages_condition': lambda messages: len(messages) > 0,
+            }
+        },
+        {
+            'name': 'Chat replay with start and end times #1',
             'params': {
                 'url': 'https://www.reddit.com/rpan/r/pan/lmvsbl',
                 'start_time': 123,
@@ -138,7 +147,17 @@ class RedditChatDownloader(BaseChatDownloader):
                 'messages_condition': lambda messages: len(messages) > 0,
             }
         },
-
+        {
+            'name': 'Chat replay with start and end times #2',
+            'params': {
+                'url': 'https://www.reddit.com/rpan/r/pan/lmvsbl',
+                'start_time': 600,
+                'end_time': 900,
+            },
+            'expected_result': {
+                'messages_condition': lambda messages: len(messages) > 0,
+            }
+        },
         # Anomalies
         {
             'name': 'Connection issues',
@@ -154,14 +173,29 @@ class RedditChatDownloader(BaseChatDownloader):
                 'timeout': 5
             }
         },
+
+        # Subreddits:
+        {
+            'name': 'Get chat of top livestream/post for subreddit #1',
+            'params': {
+                'url': 'https://www.reddit.com/rpan/r/pan',
+                'timeout': 5
+            }
+        },
+        {
+            'name': 'Get chat of top livestream/post for subreddit #2',
+            'params': {
+                'url': 'https://www.reddit.com/r/RedditSessions',
+                'timeout': 5
+            }
+        }
     ]
 
     # Regex provided by youtube-dl
 
     _VALID_URLS = {
+        '_get_chat_by_subreddit_id': r'(?:https?://(?:[^/]+\.))reddit\.com/(?:(?:rpan/r|r)/)+(?P<id>[^/?#&\s]+)/?$',
         '_get_chat_by_post_id': r'(?:https?://(?:[^/]+\.))reddit\.com/(?:(?:rpan/r/[^/]+|rpan|r/[^/]+/comments|rpan|comments)/)*(?P<id>[^/?#&\s]+)',
-        # TODO add support for "top livestream of subreddit"
-        # https://www.reddit.com/rpan/r/AnimalsOnReddit
     }
 
     _REMAPPING = {
@@ -285,35 +319,30 @@ class RedditChatDownloader(BaseChatDownloader):
 
         return info
 
+    def _get_chat_by_subreddit_id(self, match, params):
+        match_id = match.group('id')
+        return self.get_chat_by_subreddit_id(match_id, params)
+
     def _get_chat_by_post_id(self, match, params):
         match_id = match.group('id')
         return self.get_chat_by_post_id(match_id, params)
 
-    def _try_get_info(self, url, max_attempts, retry_timeout=None, **kwargs):
-
+    def _try_get_info(self, url, program_params, **kwargs):
+        max_attempts = program_params.get('max_attempts')
         for attempt_number in attempts(max_attempts):
             try:
                 return self._session_get_json(url, **kwargs)
             except (JSONDecodeError, RequestException) as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout)
-
-    def _try_post_info(self, url, max_attempts, retry_timeout=None, **kwargs):
-        for attempt_number in attempts(max_attempts):
-            try:
-                return self._session_post(url, **kwargs).json()
-            except (JSONDecodeError, RequestException) as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout)
+                self.retry(attempt_number, error=e, **program_params)
 
     _API_TEMPLATE = 'https://strapi.reddit.com/videos/t3_{}'
     _FALLBACK_API_TEMPLATE = 'https://gateway.reddit.com/desktopapi/v1/postcomments/t3_{}?limit=1'
 
-    def get_chat_by_post_id(self, post_id, params, attempt_number=0):
+    def get_chat_by_post_id(self, post_id, params, attempt_number=0, initial_info=None):
 
-        max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
-
-        initial_info = self._try_get_info(
-            self._API_TEMPLATE.format(post_id), max_attempts, retry_timeout, headers=self.authed_headers)
+        if initial_info is None:  # Optimisation
+            initial_info = self._try_get_info(self._API_TEMPLATE.format(
+                post_id), params, headers=self.authed_headers)
 
         status = initial_info.get('status')
         status_message = initial_info.get('status_message')
@@ -347,9 +376,10 @@ class RedditChatDownloader(BaseChatDownloader):
 
             if is_live and socket_url:  # live stream
 
-                if not socket_url.startswith('wss://'):
-                    raise RedditError(
-                        'Invalid websocket URL: {}'.format(socket_url))
+                if not socket_url.startswith('wss://') or 'wss.redditmedia.com' not in socket_url:
+                    self.retry(attempt_number, text='Invalid websocket URL: {}'.format(
+                        socket_url), **params)
+                    return self.get_chat_by_post_id(post_id, params, attempt_number + 1)
 
                 chat_item.chat = self._get_chat_messages_by_socket(
                     socket_url, params)
@@ -363,23 +393,21 @@ class RedditChatDownloader(BaseChatDownloader):
         elif status == 'failure':
             if 'wait' in data.lower():
                 message = 'Response from Reddit: "{}"'.format(data)
-                self.retry(attempt_number, max_attempts,
-                           retry_timeout=retry_timeout, text=message)
+                self.retry(attempt_number, text=message, **params)
                 return self.get_chat_by_post_id(post_id, params, attempt_number + 1)
 
             raise RedditError(data)
 
         elif status == 'video not found':
-            raise VideoNotFound(status_message)
+            raise VideoNotFound('Video not found')
 
-        else:  # Unknown status
-            raise UnexpectedError('Status: {} | {}'.format(status, data))
+        else:  # Unknown
+            raise UnexpectedError('Info: {}'.format(initial_info))
 
     def _get_chat_messages_by_socket(self, socket_url, params):
 
         message_receive_timeout = params.get('message_receive_timeout')
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         def create_connection():
             for attempt_number in attempts(max_attempts):
@@ -391,7 +419,7 @@ class RedditChatDownloader(BaseChatDownloader):
                     ws.settimeout(message_receive_timeout)
                     return ws
                 except (ConnectionError, websocket.WebSocketException) as e:
-                    self.retry(attempt_number, max_attempts, e, retry_timeout)
+                    self.retry(attempt_number, error=e, **params)
 
         ws = create_connection()
 
@@ -434,12 +462,9 @@ class RedditChatDownloader(BaseChatDownloader):
 
     def _get_chat_messages_by_post_id(self, post_id, params, stream_start_time=None):
 
-        max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
-
         # 1. Get all comment ids
         url = self._COMMENTS_API_TEMPLATE.format(post_id)
-        initial_info = self._try_get_info(url, max_attempts, retry_timeout)
+        initial_info = self._try_get_info(url, params)
 
         children = multi_get(initial_info, -1, 'data', 'children')
 
@@ -469,7 +494,7 @@ class RedditChatDownloader(BaseChatDownloader):
         def _parse_chunk(index):
             if not all_stored[index]:  # get if not stored
                 url = info_api + ',t1_'.join(chunk_info[index])
-                info = self._try_get_info(url, max_attempts, retry_timeout)
+                info = self._try_get_info(url, params)
                 children = multi_get(info, 'data', 'children') or []
                 all_stored[index] = [self._parse_item(
                     child.get('data'), stream_start_time) for child in children]
@@ -527,7 +552,43 @@ class RedditChatDownloader(BaseChatDownloader):
 
             log('debug', 'Total number of messages: {}'.format(count))
 
+    _SUBREDDIT_BROADCAST_API_URL = 'https://strapi.reddit.com/r/{}/broadcasts?page_size=1'
+
+    def get_chat_by_subreddit_id(self, subreddit_id, params, attempt_number=0):
+        # Get chat of top broadcast
+        initial_info = self._try_get_info(self._SUBREDDIT_BROADCAST_API_URL.format(
+            subreddit_id), params, headers=self.authed_headers)
+
+        status = initial_info.get('status')
+        data = initial_info.get('data')
+
+        if status == 'success':
+            post_id = initial_info.get('next_cursor')
+
+            data = initial_info.pop('data', [])
+            if not data:
+                raise RedditError('This subreddit has no broadcasts.')
+
+            initial_info['data'] = data[0]
+
+            return self.get_chat_by_post_id(post_id, params, initial_info=initial_info)
+
+        elif status == 'failure':
+            if 'wait' in data.lower():
+                message = 'Response from Reddit: "{}"'.format(data)
+                self.retry(attempt_number, text=message, **params)
+                return self.get_chat_by_subreddit_id(post_id, params, attempt_number + 1)
+
+            raise RedditError(data)
+
+        else:  # Unknown
+            raise UnexpectedError('Info: {}'.format(initial_info))
+
+    def _get_chat_messages_by_subreddit_id(self, subreddit_id, params):
+        pass
+
     _BROADCAST_API_URL = 'https://strapi.reddit.com/broadcasts'  # ?page_size=x
+
     _RPAN_API_URL = 'https://www.reddit.com/r/{}/new.json'
 
     # RPAN subreddits
@@ -546,6 +607,9 @@ class RedditChatDownloader(BaseChatDownloader):
         # TODO add options for live and past
 
         max_attempts = 30  # TODO make param
+        program_params = {
+            'max_attempts': max_attempts
+        }
 
         # Get live streams
         for attempt_number in attempts(max_attempts):
@@ -566,7 +630,8 @@ class RedditChatDownloader(BaseChatDownloader):
             except (JSONDecodeError, RequestException) as e:
                 error = e
 
-            self.retry(attempt_number, max_attempts, error, text=message)
+            self.retry(attempt_number, error=error,
+                       text=message, **program_params)
 
         for stream in data:
             yield multi_get(stream, 'post', 'url')
@@ -585,7 +650,7 @@ class RedditChatDownloader(BaseChatDownloader):
         count = 0
         while True:
             rpan_info = self._try_get_info(
-                api_url, max_attempts, params=past_params)
+                api_url, program_params, params=past_params)
 
             rpan_data = rpan_info.get('data')
             if not rpan_data:
