@@ -989,7 +989,7 @@ class TwitchChatDownloader(BaseChatDownloader):
         # 'contentTags': 'tags'
     }
 
-    def get_user_videos(self, username, limit=30, video_type=None, sort='TIME'):
+    def get_user_videos(self, username, limit=None, video_type=None, sort='TIME'):
         # Name -> broadcastType
 
         # Collections (ChannelCollectionsContent)
@@ -1004,9 +1004,12 @@ class TwitchChatDownloader(BaseChatDownloader):
         # Sort -> videoSort
         # VIEWS
         # TIME
+        if limit is None:
+            limit = float('inf')
 
         remaining_count = limit
-        # offset = 0
+        cursor = None
+
         while True:
             num_to_get = max(min(remaining_count, 30), 0)  # in this call
             if num_to_get <= 0:
@@ -1021,6 +1024,8 @@ class TwitchChatDownloader(BaseChatDownloader):
                     'videoSort': sort
                 }
             }]
+            if cursor:
+                query[0]['variables']['cursor'] = cursor
 
             info = self._download_gql(query)
             if not info:
@@ -1035,7 +1040,10 @@ class TwitchChatDownloader(BaseChatDownloader):
             remaining_count -= len(edges)
 
             for edge in edges:
-                node = edge['node'] or {}
+                cursor = edge.get('cursor')
+                node = edge.get('node')
+                if not node:
+                    continue
                 yield r.remap_dict(node, TwitchChatDownloader._VIDEO_REMAPPING)
 
             if not videos['pageInfo']['hasNextPage']:
@@ -1149,7 +1157,6 @@ class TwitchChatDownloader(BaseChatDownloader):
         # print('end', end_time)
 
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         messages_groups_to_add = params.get('message_groups') or []
         messages_types_to_add = params.get('message_types') or []
@@ -1168,7 +1175,7 @@ class TwitchChatDownloader(BaseChatDownloader):
                     info = self._session_get_json(url)
                     break
                 except (JSONDecodeError, RequestException) as e:
-                    self.retry(attempt_number, max_attempts, e, retry_timeout)
+                    self.retry(attempt_number, error=e, **params)
 
             error_message = multi_get(info, 'error', 'message')
 
@@ -1226,7 +1233,6 @@ class TwitchChatDownloader(BaseChatDownloader):
 
     def get_chat_by_vod_id(self, vod_id, params):
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         query = [{
             'operationName': 'VideoMetadata',
@@ -1241,7 +1247,7 @@ class TwitchChatDownloader(BaseChatDownloader):
                 video = self._download_gql(query)[0]['data']['video']
                 break
             except (JSONDecodeError, RequestException) as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout)
+                self.retry(attempt_number, error=e, **params)
 
         if not video:
             raise VideoUnavailable(
@@ -1268,7 +1274,6 @@ class TwitchChatDownloader(BaseChatDownloader):
     def get_chat_by_clip_id(self, clip_id, params):
 
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         query = {
             'query': '{ clip(slug: "%s") { broadcaster { id } video { id createdAt } createdAt durationSeconds videoOffsetSeconds title url slug } }' % clip_id,
@@ -1279,7 +1284,7 @@ class TwitchChatDownloader(BaseChatDownloader):
                 clip = self._download_base_gql(query)['data']['clip']
                 break
             except (JSONDecodeError, RequestException) as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout)
+                self.retry(attempt_number, error=e, **params)
 
         vod_id = multi_get(clip, 'video', 'id')
         # print(clip)
@@ -1514,7 +1519,6 @@ class TwitchChatDownloader(BaseChatDownloader):
 
     def _get_chat_messages_by_stream_id(self, stream_id, params):
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         message_receive_timeout = params.get('message_receive_timeout')
 
@@ -1531,7 +1535,7 @@ class TwitchChatDownloader(BaseChatDownloader):
                     irc.join_channel(stream_id)
                     return irc
                 except (socket.gaierror, ConnectionRefusedError) as e:
-                    self.retry(attempt_number, max_attempts, e, retry_timeout)
+                    self.retry(attempt_number, error=e, **params)
 
         twitch_chat_irc = create_connection()
 
@@ -1658,7 +1662,6 @@ class TwitchChatDownloader(BaseChatDownloader):
     def get_chat_by_stream_id(self, stream_id, params):
 
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         query = [{
             'operationName': 'StreamMetadata',
@@ -1670,7 +1673,7 @@ class TwitchChatDownloader(BaseChatDownloader):
                 stream_info = self._download_gql(query)[0]['data']['user']
                 break
             except (JSONDecodeError, RequestException) as e:
-                self.retry(attempt_number, max_attempts, e, retry_timeout)
+                self.retry(attempt_number, error=e, **params)
 
         if not stream_info:
             raise UserNotFound('Unable to find user: "{}"'.format(stream_id))
