@@ -9,6 +9,7 @@ from .common import (
 from ..errors import (
     ChatDownloaderError,
     NoChatReplay,
+    ChatDisabled,
     NoContinuation,
     ParsingError,
     VideoUnavailable,
@@ -41,6 +42,7 @@ from ..utils.core import (
 
 from ..debugging import (log, debug_log)
 
+import json
 import time
 import random
 import re
@@ -115,9 +117,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'timeout': 5
             }
         },
-
-
-
         {
             'name': 'Get chat messages from live chat replay',
             'params': {
@@ -125,6 +124,33 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'max_messages': 10
             },
 
+            'expected_result': {
+                'message_types': ['text_message'],
+                'action_types': ['add_chat_item'],
+                'messages_condition': lambda messages: 0 < len(messages) <= 10,
+            }
+        },
+        {
+            'name': 'Get chat messages from live chat replay',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=wXspodtIxYU',
+                'max_messages': 10
+            },
+
+            'expected_result': {
+                'message_types': ['text_message'],
+                'action_types': ['add_chat_item'],
+                'messages_condition': lambda messages: 0 < len(messages) <= 10,
+            }
+        },
+        {
+            'name': 'Get top chat messages from live chat replay',  # Premiere
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=wXspodtIxYU',
+                'start_time': 0,
+                'end_time': 20,
+                'chat_type': 'top'
+            },
             'expected_result': {
                 'message_types': ['text_message'],
                 'action_types': ['add_chat_item'],
@@ -138,7 +164,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'end_time': 20,
                 'message_groups': ['superchat', 'tickers']
             },
-
             'expected_result': {
                 'message_types': ['paid_message', 'ticker_paid_message_item', 'membership_item', 'ticker_sponsor_item', 'paid_sticker', 'ticker_paid_sticker_item'],
                 'action_types': ['add_chat_item', 'add_live_chat_ticker_item'],
@@ -152,7 +177,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'end_time': 50,
                 'message_types': ['all']
             },
-
             'expected_result': {
                 'message_types': ['viewer_engagement_message', 'paid_message', 'ticker_paid_message_item', 'text_message', 'paid_sticker', 'ticker_paid_sticker_item'],
                 'action_types': ['add_chat_item', 'add_live_chat_ticker_item'],
@@ -160,14 +184,12 @@ class YouTubeChatDownloader(BaseChatDownloader):
             }
         },
         {
-            'name': 'Get messages from top chat replay',
+            'name': 'Get messages from a premiere',  # Premiere
             'params': {
                 'url': 'https://www.youtube.com/watch?v=zVCs9Cug_qM',
                 'start_time': 0,
                 'end_time': 20,
-                'chat_type': 'top'
             },
-
             'expected_result': {
                 'message_types': ['text_message'],
                 'action_types': ['add_chat_item'],
@@ -181,9 +203,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'start_time': 0,
                 'end_time': 40,
                 'message_groups': ['donations']
-
             },
-
             'expected_result': {
                 'message_types': ['donation_announcement'],
                 'action_types': ['add_chat_item'],
@@ -198,7 +218,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'start_time': 10,
                 'end_time': 100,
             },
-
             'expected_result': {
                 'message_types': ['text_message'],
                 'action_types': ['add_chat_item'],
@@ -206,7 +225,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
             }
         },
 
-
         # TESTING FOR ERRORS
         {
             'name': 'Video does not exist',
@@ -217,6 +235,44 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'error': VideoUnavailable,
             }
         },
+        {
+            'name': 'This video is no longer available because the YouTube account associated with this video has been terminated.',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=V36LpHqtcDY',
+            },
+            'expected_result': {
+                'error': VideoUnavailable,
+            }
+        },
+        {
+            'name': 'This video is not available.',  # YouTube Premium
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=i1Ko8UG-Tdo',
+            },
+            'expected_result': {
+                'error': VideoUnplayable,
+            }
+        },
+        {
+            'name': 'This video is not available.',  # Rental video preview
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=yYr8q0y5Jfg',
+            },
+            'expected_result': {
+                'error': VideoUnplayable,
+            }
+        },
+        {
+            # The following content has been identified by the YouTube community
+            # as inappropriate or offensive to some audiences.
+            'name': "This video has been removed for violating YouTube's policy on hate speech. Learn more about combating hate speech in your country.",  # Rental video preview
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=6SJNVb0GnPI',
+            },
+            'expected_result': {
+                'error': VideoUnavailable,
+            }
+        },
         {
             'name': 'Members-only content',
             'params': {
@@ -232,7 +288,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'url': 'https://www.youtube.com/watch?v=XWq5kBlakcQ',
             },
             'expected_result': {
-                'error': NoChatReplay,
+                'error': ChatDisabled,
             }
         },
         {
@@ -244,6 +300,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'error': NoChatReplay,
             }
         },
+
         {
             'name': 'Video is private',
             'params': {
@@ -262,6 +319,25 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'error': VideoUnplayable,
             }
         },
+        {
+            'name': 'This live stream recording is not available.',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=qEJwOuvDf7I',
+            },
+            'expected_result': {
+                'error': VideoUnplayable,
+            }
+        },
+        {
+            # Age restricted
+            'name': 'Sign in to confirm your age. This video may be inappropriate for some users.',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=WaOKSUlf4TM',
+            },
+            'expected_result': {
+                'error': LoginRequired,
+            }
+        },
 
         # Potential parsing errors
         {
@@ -404,7 +480,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
                     (?P<id>[a-zA-Z0-9_-]+)'''
     }
 
-    @staticmethod
+    @ staticmethod
     def _get_source_image_url(url):
         index = url.find('=')
         if index >= 0:
@@ -625,7 +701,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
             'text': multi_get(item, 'buttonRenderer', 'text', 'simpleText') or ''
         }
 
-    @staticmethod
+    @ staticmethod
     def _get_simple_text(item):
         return item.get('simpleText')
 
@@ -653,7 +729,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
     # https://en.wikipedia.org/wiki/ISO_4217
     # e.g. 'CHF', 'COP', 'HUF', 'PHP', 'PLN', 'RUB', 'SEK', 'PEN', 'ARS', 'CLP', 'NOK', 'BAM', 'SGD'
 
-    @staticmethod
+    @ staticmethod
     def _parse_currency(item):
         mixed_text = item.get('simpleText') or str(item)
 
@@ -934,8 +1010,11 @@ class YouTubeChatDownloader(BaseChatDownloader):
     _LIVE_PLAYLIST_URL = _YT_HOME + '/channel/UC4R8DWoMoI7CAwX8_LjQHig'
 
     def _get_testing_items(self):
+        params = {
+            'max_attempts': 10
+        }
 
-        html, yt_info = self._get_initial_info(self._LIVE_PLAYLIST_URL)
+        html, yt_info = self._get_initial_info(self._LIVE_PLAYLIST_URL, params)
 
         sections = yt_info['contents']['twoColumnBrowseResultsRenderer']['tabs'][
             0]['tabRenderer']['content']['sectionListRenderer']['contents']
@@ -955,7 +1034,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
             yield from self.get_playlist_items(playlist_url)
 
-    @staticmethod
+    @ staticmethod
     def _get_rendered_content(yt_info, tab_index=0):
         return yt_info['contents']['twoColumnBrowseResultsRenderer']['tabs'][tab_index]['tabRenderer']['content'][
             'sectionListRenderer']['contents'][0]['itemSectionRenderer']['contents'][0]
@@ -969,7 +1048,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         # 'videoId', 'thumbnail', 'title', 'viewCountText', 'navigationEndpoint', 'ownerBadges', 'trackingParams', 'shortViewCountText', 'menu', 'thumbnailOverlays'
     }
 
-    @staticmethod
+    @ staticmethod
     def _parse_video(video_renderer):
         return r.remap_dict(video_renderer, YouTubeChatDownloader._VIDEO_REMAPPING)
 
@@ -979,7 +1058,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         'past': (503, 'Past live streams')
     }
 
-    def get_user_videos(self, channel_id=None, user_id=None, custom_username=None, video_type='live'):
+    def get_user_videos(self, channel_id=None, user_id=None, custom_username=None, video_type='live', params=None):
         """[summary]
         If more than one of `channel_id`, `user_id` and `custom_username`
         are specifed, the first one specified will be returned.
@@ -1017,7 +1096,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         user_url = 'https://www.youtube.com/{}/{}'.format(_type, _id)
 
         html, yt_info = self._get_initial_info(
-            '{}/videos?view=2&live_view={}'.format(user_url, vid_type[0]))
+            '{}/videos?view=2&live_view={}'.format(user_url, vid_type[0]), params)
 
         section_list_renderer = multi_get(
             yt_info, 'contents', 'twoColumnBrowseResultsRenderer', 'tabs', 1, 'tabRenderer', 'content', 'sectionListRenderer')
@@ -1048,9 +1127,9 @@ class YouTubeChatDownloader(BaseChatDownloader):
             else:
                 print(item)
 
-    def get_playlist_items(self, playlist_url):
+    def get_playlist_items(self, playlist_url, params=None):
 
-        html, yt_info = self._get_initial_info(playlist_url)
+        html, yt_info = self._get_initial_info(playlist_url, params)
 
         items = self._get_rendered_content(
             yt_info)['playlistVideoListRenderer']['contents']
@@ -1102,17 +1181,56 @@ class YouTubeChatDownloader(BaseChatDownloader):
             f'{time_now} {sapisid_cookie} {self._YT_HOME}'.encode('utf-8')).hexdigest()
         return f'SAPISIDHASH {time_now}_{sapisidhash}'
 
-    def _get_initial_info(self, url):
-        html = self._session_get(url).text
-        yt = regex_search(html, self._YT_INITIAL_DATA_RE)
-        yt_initial_data = try_parse_json(yt) if yt else None
-        return html, yt_initial_data
+    def _get_continuation_info(self, continuation_url, program_params, **post_kwargs):
+        max_attempts = program_params.get('max_attempts')
 
-    def _get_initial_video_info(self, video_id):
+        for attempt_number in attempts(max_attempts):
+            try:
+                response = self._session_post(continuation_url, **post_kwargs)
+                json_response = response.json()
+
+                # Check for errors:
+                error = json_response.get('error')
+                if error:
+                    error_code = error.get('code')
+                    error_message = error.get('message')
+
+                    if error_code // 100 == 5:  # Server error, retry
+                        self.retry(attempt_number,
+                                   text=error_message, **program_params)
+                        continue
+
+                return json_response
+
+            except JSONDecodeError as e:
+                self.retry(attempt_number, error=e, **program_params,
+                           text='Unable to parse JSON: `{}`'.format(response.text))
+
+            except RequestException as e:
+                self.retry(attempt_number, error=e, **program_params)
+
+    def _get_initial_info(self, url, params=None):
+        if params is None:
+            params = {}
+
+        max_attempts = params.get('max_attempts', 1)
+        for attempt_number in attempts(max_attempts):
+            try:
+                html = self._session_get(url).text
+                yt = regex_search(html, self._YT_INITIAL_DATA_RE)
+                yt_initial_data = json.loads(yt)
+                return html, yt_initial_data
+
+            except (JSONDecodeError, RequestException) as e:
+                self.retry(attempt_number, error=e, **params)
+
+        return None, None
+
+    def _get_initial_video_info(self, video_id, params=None):
         """ Get initial YouTube video information. """
         original_url = self._YT_VIDEO_TEMPLATE.format(video_id)
 
-        html, yt_initial_data = self._get_initial_info(original_url)
+        html, yt_initial_data = self._get_initial_info(original_url, params)
 
         if not yt_initial_data:  # Fatal error
             raise ParsingError(
@@ -1211,7 +1329,11 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 error_message = self._parse_runs(error_runs, False)[
                     'message'] if error_runs else 'Video does not have a chat replay.'
 
-                raise NoChatReplay(error_message)
+                # Live chat replay was turned off for this video. -> NoChatReplay
+                if 'disabled' in error_message:
+                    raise ChatDisabled(error_message)
+                else:
+                    raise NoChatReplay(error_message)
 
         video_details = player_response_info.get('videoDetails') or {}
         details['title'] = video_details.get('title')
@@ -1305,7 +1427,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
         # force_no_timeout = params.get('force_no_timeout')
 
         max_attempts = params.get('max_attempts')
-        retry_timeout = params.get('retry_timeout')
 
         messages_groups_to_add = params.get('message_groups') or []
         messages_types_to_add = params.get('message_types') or []
@@ -1347,62 +1468,43 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 })
 
             info = None
-            for attempt_number in attempts(max_attempts):
 
-                try:
-                    if first_time:
-                        # must run to get first few messages, otherwise might miss some
-                        html, yt_info = self._get_initial_info(init_page)
+            if first_time:
+                # must run to get first few messages, otherwise might miss some
+                html, yt_info = self._get_initial_info(init_page, params)
 
-                    else:
-                        if not is_live and offset_milliseconds is not None:
-                            continuation_params['currentPlayerState'] = {
-                                'playerOffsetMs': offset_milliseconds}
+            else:
+                if not is_live and offset_milliseconds is not None:
+                    continuation_params['currentPlayerState'] = {
+                        'playerOffsetMs': offset_milliseconds}
 
-                        if click_tracking_params:
-                            continuation_params['context']['clickTracking'] = {
-                                'clickTrackingParams': click_tracking_params}
+                if click_tracking_params:
+                    continuation_params['context']['clickTracking'] = {
+                        'clickTrackingParams': click_tracking_params}
 
-                        yt_info = self._session_post(
-                            continuation_url, json=continuation_params).json()
+                yt_info = self._get_continuation_info(
+                    continuation_url, params, json=continuation_params)
 
-                    debug_info = {
-                        'click_tracking': multi_get(continuation_params, 'context', 'clickTracking'),
-                        'continuation': multi_get(continuation_params, 'continuation')
-                    }
-                    log('debug', 'Continuation parameters: {}'.format(debug_info))
-                    log('debug', 'Session headers: {}'.format(
-                        ', '.join(self.session.headers.keys())))  # Only display keys
+            debug_info = {
+                'click_tracking': multi_get(continuation_params, 'context', 'clickTracking'),
+                'continuation': multi_get(continuation_params, 'continuation')
+            }
+            log('debug', 'Continuation parameters: {}'.format(debug_info))
+            log('debug', 'Session headers: {}'.format(
+                ', '.join(self.session.headers.keys())))  # Only display keys
 
-                    info = multi_get(
-                        yt_info, 'continuationContents', 'liveChatContinuation')
+            info = multi_get(
+                yt_info, 'continuationContents', 'liveChatContinuation')
 
-                    logged_in_info = multi_get(
-                        yt_info, 'responseContext', 'serviceTrackingParams', 1, 'params', 0)
-                    log('debug', 'Logged-in info: {}'.format(logged_in_info))
+            logged_in_info = multi_get(
+                yt_info, 'responseContext', 'serviceTrackingParams', 1, 'params', 0)
+            log('debug', 'Logged-in info: {}'.format(logged_in_info))
 
-                    if not info:
-                        log('debug', 'No continuation information found: {}'.format(
-                            yt_info))
+            if not info:
+                log('debug', 'No continuation information found: {}'.format(
+                    yt_info))
 
-                        # Check for errors:
-                        error = yt_info.get('error')
-                        if error:
-                            error_code = error.get('code')
-                            error_message = error.get('message')
-
-                            if error_code // 100 == 5:  # Server error, retry
-                                self.retry(
-                                    attempt_number, max_attempts, retry_timeout=retry_timeout, text=error_message)
-                                continue
-
-                        return
-
-                    break  # successful retrieve
-
-                except (JSONDecodeError, RequestException) as e:
-                    self.retry(attempt_number, max_attempts, e, retry_timeout)
-                    continue
+                return
 
             actions = info.get('actions') or []
 
@@ -1669,9 +1771,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
             if first_time:
                 first_time = False
 
-    # def get_chat_by_user(self, channel_id=None, user_id=None, custom_username=None):
-    #     def get_user_videos(self, video_type='live'):
-
     def _get_chat_by_user(self, match, params):
         match_id = match.group('id')
         user_type = match.group('type')  # channel|c|user
@@ -1728,7 +1827,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         while True:
             for video_type in ('live', 'upcoming'):
                 # prioritise live videos
-                for video in self.get_user_videos(**user_video_args, video_type=video_type):
+                for video in self.get_user_videos(**user_video_args, video_type=video_type, params=params):
                     video_id = video['video_id']
                     video_title = video['title']
 
@@ -1773,7 +1872,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         :return: Chat object for the corresponding YouTube video
         :rtype: Chat
         """
-        initial_info = self._get_initial_video_info(video_id)
+        initial_info = self._get_initial_video_info(video_id, params)
 
         title = initial_info.get('title')
         duration = initial_info.get('duration')
